unit CrossCryptMain;



// xxx - intercept windows "drive arrived" message - like SecureTrayUtil does


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Spin, StdCtrls, ComCtrls, ImgList, ExtCtrls, Grids, Menus, OTFE_U,
  OTFECrossCrypt_U;

type
  TCrossCryptMain_F = class(TForm)
    ilDriveIcons: TImageList;
    lvDrives: TListView;
    pmDrives: TPopupMenu;
    pmDismount: TMenuItem;
    OTFECrossCrypt: TOTFECrossCrypt;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    miMount: TMenuItem;
    miDismount: TMenuItem;
    miExit: TMenuItem;
    N2: TMenuItem;
    pmProperties: TMenuItem;
    N3: TMenuItem;
    miCreate: TMenuItem;
    View1: TMenuItem;
    miProperties: TMenuItem;
    N1: TMenuItem;
    miRefresh: TMenuItem;
    OpenDialog: TOpenDialog;
    Help1: TMenuItem;
    About1: TMenuItem;
    miDismountAll: TMenuItem;
    pmDismountAll: TMenuItem;
    procedure miExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure miRefreshClick(Sender: TObject);
    procedure miCreateClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure lvDrivesResize(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure miMountClick(Sender: TObject);
    procedure miPropertiesClick(Sender: TObject);
    procedure pmPropertiesClick(Sender: TObject);
    procedure miDismountClick(Sender: TObject);
    procedure pmDismountClick(Sender: TObject);
    procedure pmDismountAllClick(Sender: TObject);
    procedure miDismountAllClick(Sender: TObject);
    procedure lvDrivesClick(Sender: TObject);
    procedure lvDrivesDblClick(Sender: TObject);
    procedure lvDrivesChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);

  private
    procedure InitializeDrives();
    procedure RefreshDrives();
    function  AddIconForDrive(driveLetter: char): integer;

    procedure EnableDisableControls();
    function  GetDriveLetterFromLVItem(listItem: TListItem): char;

    procedure ResizeWindow();

    procedure MountFiles(filenames: TStringList; readOnly: boolean);

    procedure DriveProperties();

    procedure DismountSelected();
    procedure DismountAll();
    procedure ReportDrivesNotDismounted(drivesRemaining: string);

  public
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
    procedure WMDeviceChange(var Msg : TMessage); message WM_DEVICECHANGE;
  end;

var
  CrossCryptMain_F: TCrossCryptMain_F;

implementation

{$R *.DFM}

uses
  ShellApi,  // Required for SHGetFileInfo
  Commctrl, // required for ImageList_GetIcon
  About_U,
  NewVolumeSize,
  CrossCryptVolProperties,
  OTFEConsts_U;

const
  CRLF = #13+#10;


// Format disk related functions ripped from the Unofficial Delphi FAQ (UDF)
const
        SHFMT_ID_DEFAULT        = $FFFF;
        // Formating options
        SHFMT_OPT_QUICKFORMAT   = $0000;
        SHFMT_OPT_FULL          = $0001;
        SHFMT_OPT_SYSONLY       = $0002;
        // Error codes
        SHFMT_ERROR             = $FFFFFFFF;
        SHFMT_CANCEL            = $FFFFFFFE;
        SHFMT_NOFORMAT          = $FFFFFFFD;

function SHFormatDrive(Handle: HWND; Drive, ID, Options: Word): LongInt;
            stdcall; external 'shell32.dll' name 'SHFormatDrive'


procedure TCrossCryptMain_F.RefreshDrives();
var
  ListItem: TListItem;
  driveIconNum: integer;
  i: integer;
  volumeInfo: TCrossCryptVolumeInfo;
  mounted: string;
begin
  // Cleardown all...
  lvDrives.Items.Clear();
  ilDriveIcons.Clear();

  mounted := OTFECrossCrypt.DrivesMounted();

  for i:=1 to length(mounted) do
    begin
    if OTFECrossCrypt.GetVolumeInfo(mounted[i], volumeInfo) then
      begin
      ListItem := lvDrives.Items.Add;
      ListItem.Caption := '     '+mounted[i]+':';
      ListItem.SubItems.Add(volumeInfo.Filename);
      driveIconNum := AddIconForDrive(mounted[i]);
      if (driveIconNum>=0) then
        begin
        ListItem.ImageIndex := driveIconNum;
        end;
      end
    else
      begin
      // Strange... The drive probably got dismounted between getting the
      // list of mounted drives, and querying them for more information...
      end;

    end;

end;


// Add an icon to represent the specified drive
// Returns: The icon's index in ilDriveIcons, or -1 on error
function TCrossCryptMain_F.AddIconForDrive(driveLetter: char): integer;
var
  iconIdx: integer;
  anIcon: TIcon;
  shfi: SHFileInfo;
  imgListHandle: THandle;
  iconHandle: HICON;
  tmpDrivePath: string;
begin
  iconIdx := -1;
  
  // Create a suitable icon to represent the drive
  tmpDrivePath := driveLetter+':\';
  imgListHandle := SHGetFileInfo(
                                 PChar(tmpDrivePath),
                                 0,
                                 shfi,
                                 sizeof(shfi),
                                 SHGFI_SYSICONINDEX or SHGFI_LARGEICON);
//                                 SHGFI_DISPLAYNAME

  if imgListHandle<>0 then
    begin
    iconHandle := ImageList_GetIcon(imgListHandle, shfi.iIcon, ILD_NORMAL);
    if (iconHandle<>0) then
      begin
      anIcon:= TIcon.Create();
      try
        anIcon.ReleaseHandle();

        anIcon.handle := iconHandle;

        if (ilDriveIcons.Width<anIcon.Width) then
          begin
          ilDriveIcons.Width := anIcon.Width;
          end;
        if (ilDriveIcons.Height<anIcon.Height) then
          begin
          ilDriveIcons.Height := anIcon.Height;
          end;

        iconIdx := ilDriveIcons.addicon(anIcon);

        anIcon.ReleaseHandle();
        DestroyIcon(iconHandle);
      finally
        anIcon.Free();
      end;

      end;
    end;


  Result := iconIdx;

end;

procedure TCrossCryptMain_F.InitializeDrives();
var
  tmpColumn: TListColumn;
begin
  // Listview setup...
  lvDrives.SmallImages := ilDriveIcons;
  lvDrives.RowSelect:= TRUE;
  lvDrives.MultiSelect:= TRUE;
  lvDrives.showcolumnheaders := TRUE;
  lvDrives.ReadOnly := TRUE;
  lvDrives.ViewStyle := vsReport;
//    lvDrives.ViewStyle := vsIcon;
//    lvDrives.ViewStyle := vsList;
  lvDrives.IconOptions.Arrangement := iaTop;


  // Listview columns...
  tmpColumn := lvDrives.Columns.Add;
  tmpColumn.Caption := 'Drive';
  tmpColumn.width := 100;

//    tmpColumn.minwidth := lvDrives.width;
  tmpColumn := lvDrives.Columns.Add;
  tmpColumn.Caption := 'Volume';
  tmpColumn.width := lvDrives.clientwidth - lvDrives.columns[0].width - ilDriveIcons.width;


// xxx - shouldn't this work?     NewColumn.autosize := TRUE;
// xxx - shouldn't this work?     NewColumn.minwidth := lvDrives.clientwidth - lvDrives.columns[0].width - ilDriveIcons.width;


// xxx - use destroyimage or whatever it was to cleardown the **imagelist**

end;


procedure TCrossCryptMain_F.miExitClick(Sender: TObject);
begin
  Close();
  
end;

procedure TCrossCryptMain_F.FormCreate(Sender: TObject);
begin
  try
    OTFECrossCrypt.Active := TRUE;
  except
    // Damn. We shouldn't get to here :(
  end;


  if (not(OTFECrossCrypt.Active)) then
    begin
    MessageDlg(
               'Unable to initialize support for CrossCrypt.'+CRLF+
               CRLF+
               'Please ensure that CrossCrypt is installed; if you have only'+CRLF+
               'just installed CrossCrypt, please reboot and try again.',
               mtError,
               [mbOK],
               0
              );

    Application.Terminate();
    exit;
    end;

  InitializeDrives();

  // Let Windows know we accept dropped files
  DragAcceptFiles(self.Handle, True);

end;

procedure TCrossCryptMain_F.miRefreshClick(Sender: TObject);
begin
  RefreshDrives();

end;

procedure TCrossCryptMain_F.miCreateClick(Sender: TObject);
var
  volSizeDlg: TNewVolumeSize_F;
  newDrive: char;
  msgDlgResponse: word;
  driveNum: word;
  errMsg: string;
begin
  volSizeDlg:= TNewVolumeSize_F.Create(nil);
  try
    if (volSizeDlg.ShowModal = mrOK) then
      begin
      newDrive := OTFECrossCrypt.CreateVolume(volSizeDlg.Filename, volSizeDlg.VolumeSize);
      if (newDrive=#0) then
        begin
        if (OTFECrossCrypt.LastErrorCode <> OTFE_ERR_USER_CANCEL) then
          begin
          errMsg := 'An error occured while trying to create your volume file';
          if (OTFECrossCrypt.LastErrorCode = OTFE_ERR_NO_FREE_DEVICES) then
            begin
            errMsg := 'Your volume could not be created at this time as there are'+CRLF+
                      'no suitable CrossCrypt devices free.'+CRLF+
                      CRLF+
                      'Please dismount one or more volumes, and try again.';
            end;

          MessageDlg(errMsg, mtError, [mbOK], 0);
          end;
        end
      else
        begin
        msgDlgResponse := MessageDlg('The new volume has been created, and mounted as drive '+newDrive+':'+CRLF+
                                     CRLF+
                                     'Before this drive can be used, it must be formatted first.'+
                                     CRLF+'Do you wish to format this volume now?',
                                     mtConfirmation,
                                     [mbYes, mbNo],
                                     0);
        if (msgDlgResponse=mrYes) then
          begin
          driveNum := ord(newDrive) - ord('A');
          SHFormatDrive(Handle, driveNum, SHFMT_ID_DEFAULT, SHFMT_OPT_FULL);
          end;

        RefreshDrives();
        end;
      end;

  finally
    volSizeDlg.Free();
  end;

end;

procedure TCrossCryptMain_F.About1Click(Sender: TObject);
var
  aboutDlg: TAbout_F;
begin
  aboutDlg:= TAbout_F.Create(nil);
  try
    aboutDlg.OTFECrossCrypt := OTFECrossCrypt;
    aboutDlg.ShowModal();
  finally
    aboutDlg.Free();
  end;

end;

procedure TCrossCryptMain_F.lvDrivesResize(Sender: TObject);
begin
  ResizeWindow();

end;


procedure TCrossCryptMain_F.FormResize(Sender: TObject);
begin
  ResizeWindow();

end;



// Resize the columns in the drives list so the 2nd column fills out...
procedure TCrossCryptMain_F.ResizeWindow();
var
  tmpColumn: TListColumn;
begin
  if lvDrives.columns.count>1 then
    begin
    tmpColumn := lvDrives.columns[1];
    tmpColumn.width := lvDrives.clientwidth - lvDrives.columns[0].width - ilDriveIcons.width;
    end;

end;


procedure TCrossCryptMain_F.FormShow(Sender: TObject);
begin
  RefreshDrives();
  EnableDisableControls();

end;


procedure TCrossCryptMain_F.MountFiles(filenames: TStringList; readOnly: boolean);
var
  i: integer;
  mountedAs: string;
  msg: string;
  errMsg: string;
begin
  if not(OTFECrossCrypt.Mount(filenames, mountedAs, readOnly)) then
    begin
    if (OTFECrossCrypt.LastErrorCode <> OTFE_ERR_USER_CANCEL) then
      begin
      errMsg := 'An error occured while trying to mount your volume file';
      if (OTFECrossCrypt.LastErrorCode = OTFE_ERR_NO_FREE_DEVICES) then
        begin
        errMsg := 'Your volume could not be mounted at this time as there are'+CRLF+
                  'no suitable CrossCrypt devices free.'+CRLF+
                  CRLF+
                  'Please dismount one or more volumes, and try again.';
        end;

      MessageDlg(errMsg, mtError, [mbOK], 0);
      end;

    end
  else
    begin
    if (length(mountedAs)=1) then
      begin
      msg := 'Your CrossCrypt volume has been mounted as drive: ';
      end
    else
      begin
      msg := 'Your CrossCrypt volumes have been mounted as drives: ';
      end;

    for i:=1 to length(mountedAs) do
      begin
      if (i<>1) then
        begin
        msg := msg + ', ';
        end;

      msg := msg + mountedAs[i] + ':';
      end;

    RefreshDrives();
    showmessage(msg);
    end;

end;


procedure TCrossCryptMain_F.miMountClick(Sender: TObject);
begin
  if OpenDialog.Execute() then
    begin
    MountFiles(TStringList(OpenDialog.Files), (ofReadOnly in OpenDialog.Options));
    end;

end;



// Handle files being dropped onto the form from (for example) Windows Explorer.
// Files dropped onto the form should be treated as though the user is trying
// to mount them as volume files
procedure TCrossCryptMain_F.WMDropFiles(var Msg: TWMDropFiles);
var
  buffer: array[0..MAX_PATH] of Char;
  numFiles: longint;
  i: integer;
  filesToMount: TStringList;
begin
  try
    // Bring our window to the foreground
    SetForeGroundWindow(self.handle);

    // Handle the dropped files
    numFiles := DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0);
    filesToMount := TStringList.Create();
    try
      for i:=0 to (numFiles-1) do
        begin
        DragQueryFile(msg.Drop,
                      i,
                      @buffer,
                      sizeof(buffer));
        filesToMount.Add(buffer);
        end;

      MountFiles(filesToMount, FALSE);

      Msg.Result := 0;
    finally
      filesToMount.Free();
    end;
  finally
    DragFinish(Msg.Drop);
  end;

end;


procedure TCrossCryptMain_F.WMDeviceChange(var Msg : TMessage);
begin
  RefreshDrives();

end;


procedure TCrossCryptMain_F.DriveProperties();
var
  propertiesDlg: TCrossCryptVolProperties_F;
  i: integer;
begin
  for i:=0 to (lvDrives.items.count-1) do
    begin
    if lvDrives.Items[i].Selected then
      begin
      propertiesDlg:= TCrossCryptVolProperties_F.Create(nil);
      try
        propertiesDlg.DriveLetter := GetDriveLetterFromLVItem(lvDrives.Items[i]);
        propertiesDlg.OTFECrossCrypt := OTFECrossCrypt;
        propertiesDlg.ShowModal();
      finally
        propertiesDlg.Free();
      end;

      end;
    end;

end;


procedure TCrossCryptMain_F.miPropertiesClick(Sender: TObject);
begin
  DriveProperties();

end;

procedure TCrossCryptMain_F.pmPropertiesClick(Sender: TObject);
begin
  DriveProperties();

end;


procedure TCrossCryptMain_F.DismountAll();
var
  drivesRemaining: string;
begin
  OTFECrossCrypt.DismountAll(FALSE);

  drivesRemaining := OTFECrossCrypt.DrivesMounted();
  if (drivesRemaining<>'') then
    begin
    ReportDrivesNotDismounted(drivesRemaining);
    end;

  RefreshDrives();

end;


procedure TCrossCryptMain_F.DismountSelected();
var
  i: integer;
  tmpDrv: char;
  drivesRemaining: string;
  toDismount: string;
begin
  drivesRemaining := '';

  // First we build up a list of drives to dismount, then we dismount them.
  // This is done since we can't just run through the lvDrives looking for
  // selected items, as they're getting removed while we walk though the list!
  
  for i:=0 to (lvDrives.items.count-1) do
    begin
    if lvDrives.Items[i].Selected then
      begin
      toDismount := toDismount + GetDriveLetterFromLVItem(lvDrives.Items[i]);
      end;
    end;

  for i:=1 to length(toDismount) do
    begin
    tmpDrv := toDismount[i];
    if not(OTFECrossCrypt.Dismount(tmpDrv, FALSE)) then
      begin
      drivesRemaining := drivesRemaining + tmpDrv;
      end;

    end;

  if (drivesRemaining<>'') then
    begin
    ReportDrivesNotDismounted(drivesRemaining);
    end;

  RefreshDrives();

end;


procedure TCrossCryptMain_F.ReportDrivesNotDismounted(drivesRemaining: string);
var
  msg: string;
  i: integer;
begin
  msg := 'Unable to dismount drive(s): ';
  for i:=1 to length(drivesRemaining) do
    begin
    if (i<>1) then
      begin
      msg := msg + ', ';
      end;

    msg := msg + drivesRemaining[i] + ':';
    end;

  MessageDlg(msg, mtWarning, [mbOK], 0);

end;



procedure TCrossCryptMain_F.miDismountClick(Sender: TObject);
begin
  DismountSelected();
  
end;

procedure TCrossCryptMain_F.pmDismountClick(Sender: TObject);
begin
  DismountSelected();
  
end;

procedure TCrossCryptMain_F.pmDismountAllClick(Sender: TObject);
begin
  DismountAll();

end;

procedure TCrossCryptMain_F.miDismountAllClick(Sender: TObject);
begin
  DismountAll();
  
end;

procedure TCrossCryptMain_F.lvDrivesClick(Sender: TObject);
begin
  EnableDisableControls();

end;

procedure TCrossCryptMain_F.EnableDisableControls();
var
  drivesSelected: boolean;
  drivesMounted: boolean;
begin
  drivesSelected := (lvDrives.selcount > 0);
  drivesMounted := (lvDrives.Items.count > 0);
  
  miDismount.enabled := drivesSelected;
  pmDismount.enabled := drivesSelected;
  miProperties.enabled := drivesSelected;
  pmProperties.enabled := drivesSelected;

  miDismountAll.enabled := drivesMounted;
  pmDismountAll.enabled := drivesMounted;

end;


function TCrossCryptMain_F.GetDriveLetterFromLVItem(listItem: TListItem): char;
var
  tmpDrv: string;
begin
  // Trim the padding whitespace off the drive letter + colon
  tmpDrv := listItem.Caption;
  tmpDrv := TrimLeft(tmpDrv);

  // The first letter of the item's caption is the drive letter
  Result := tmpDrv[1];

end;

procedure TCrossCryptMain_F.lvDrivesDblClick(Sender: TObject);
var
  driveLetter: string;
  explorerCommandLine: string;
begin
  if (lvDrives.selcount > 0) then
    begin
    driveLetter := GetDriveLetterFromLVItem(lvDrives.selected);

    if (driveLetter<>'') then
      begin
      explorerCommandLine := 'explorer ' + driveLetter + ':\';

      if (WinExec(PChar(explorerCommandLine), SW_RESTORE))<31 then
        begin
        MessageDlg('Error running explorer', mtError, [], 0);
        end;

      end;
    end;

end;

procedure TCrossCryptMain_F.lvDrivesChange(Sender: TObject;
  Item: TListItem; Change: TItemChange);
begin
  EnableDisableControls();

end;

END.



